
/*
 * (c) Copyright 2018 CORSIKA Project, corsika-project@lists.kit.edu
 *
 * See file AUTHORS for a list of contributors.
 *
 * This software is distributed under the terms of the GNU General Public
 * Licence version 3 (GPL Version 3). See file LICENSE for a full version of
 * the license.
 */

#ifndef _corsika_process_sibyll_nuclearinteraction_h_
#define _corsika_process_sibyll_nuclearinteraction_h_

#include <corsika/particles/ParticleProperties.h>
#include <corsika/process/InteractionProcess.h>
#include <corsika/random/RNGManager.h>

namespace corsika::environment {
  class Environment;
}

namespace corsika::process::sibyll {

  class Interaction; // fwd-decl

  /**
   *
   *
   **/

  class NuclearInteraction
      : public corsika::process::InteractionProcess<NuclearInteraction> {

    int fCount = 0;
    int fNucCount = 0;

  public:
    NuclearInteraction(corsika::environment::Environment const& env,
                       corsika::process::sibyll::Interaction& hadint);
    ~NuclearInteraction();
    void Init();
    void InitializeNuclearCrossSections();
    void PrintCrossSectionTable(corsika::particles::Code);
    corsika::units::si::CrossSectionType ReadCrossSectionTable(const int, corsika::particles::Code, corsika::units::si::HEPEnergyType);
    corsika::units::si::HEPEnergyType GetMinEnergyPerNucleonCoM() { return fMinEnergyPerNucleonCoM; }
    corsika::units::si::HEPEnergyType GetMaxEnergyPerNucleonCoM() { return fMaxEnergyPerNucleonCoM; }
    int GetMaxNucleusAProjectile() {return fMaxNucleusAProjectile; }
    int GetMaxNFragments() { return fMaxNFragments; }
    int GetNEnergyBins() { return fNEnBins; }
    template <typename Particle>
    std::tuple<corsika::units::si::CrossSectionType, corsika::units::si::CrossSectionType>
    GetCrossSection(Particle& p, const corsika::particles::Code TargetId);

    template <typename Particle, typename Track>
    corsika::units::si::GrammageType GetInteractionLength(Particle& p, Track&);

    template <typename Particle, typename Stack>
    corsika::process::EProcessReturn DoInteraction(Particle& p, Stack& s);

  private:
    corsika::environment::Environment const& fEnvironment;
    corsika::process::sibyll::Interaction& fHadronicInteraction;
    std::map<corsika::particles::Code,int> fTargetComponentsIndex;
    corsika::random::RNG& fRNG =
        corsika::random::RNGManager::GetInstance().GetRandomStream("s_rndm");
    const int fNSample = 500; // number of samples in MC estimation of cross section
    const int fMaxNucleusAProjectile = 56;
    const int fNEnBins = 6;
    const int fMaxNFragments = 60;
    // energy limits defined by table used for cross section in signuc.f
    // 10**1 GeV to 10**6 GeV
    const corsika::units::si::HEPEnergyType fMinEnergyPerNucleonCoM = 
      10. * 1e9 * corsika::units::si::electronvolt;
    const corsika::units::si::HEPEnergyType fMaxEnergyPerNucleonCoM = 
      1.e6 * 1e9 * corsika::units::si::electronvolt;
  };

} // namespace corsika::process::sibyll

#endif
